import unittest
from bristol.ensembles import Circular
from bristol.spectral import Ergodicity
import numpy as np     

class test_spectral_density(unittest.TestCase):

      epsilon = 1e-9

      def test_spectral_density_01(self):
          ce          = Circular()
          mseed       = [123,125,124]       
          Ns          = [5, 10, 15]
          cSize       = 2
          nChunks     = 3
          eigen_data5 = ce.eigen_circular_ensemble(
                                                   Ns[0], 
                                                   cSize=cSize, 
                                                   nchunks=nChunks, 
                                                   seeds=mseed, 
                                                   parallel=False
                                                  )
          eigen_data10 = ce.eigen_circular_ensemble(
                                                    Ns[1], 
                                                    cSize=cSize, 
                                                    nchunks=nChunks, 
                                                    seeds=mseed, 
                                                    parallel=False
                                                   )
          eigen_data15 = ce.eigen_circular_ensemble(
                                                    Ns[2], 
                                                    cSize=cSize, 
                                                    nchunks=nChunks, 
                                                    seeds=mseed, 
                                                    parallel=False
                                                   )
          ensemble_size = cSize*nChunks 
          eigen_data    = {
                           "N5":eigen_data5, 
                           "N10":eigen_data10,
                           "N15":eigen_data15
                          }
          ergo = Ergodicity()
          Dse  = ergo.approach_se(
                                  Ns=Ns,
                                  ensemble_size=ensemble_size,
                                  eigen_data=eigen_data
                                 )
          self.assertTrue(len(Dse) == 2)
          d0 = -12.287133235052062 
          d1 = -8.3998021593200445
          self.assertTrue(Dse[0]+d0 < self.epsilon)
          self.assertTrue(Dse[1]+d1 < self.epsilon)



import unittest
from bristol.ensembles import Circular
import numpy as np

class test_eigen_circular(unittest.TestCase):

      epsilon = 1e-9

      def test_eigen_circular_01(self):
          ce      = Circular()
          mseed   = 2963416
          e_cue   = ce.eigen_circular(5,  ensemble='CUE', set_seed=True, seed=mseed)  
          e_coe   = ce.eigen_circular(10, ensemble='COE', set_seed=True, seed=mseed)  
          e_cse   = ce.eigen_circular(15, ensemble='CSE', set_seed=True, seed=mseed)  
          n_cue   = np.imag(e_cue).sum()
          n_coe   = np.imag(e_coe).sum()
          n_cse   = np.imag(e_cse).sum()
          self.assertTrue(n_cue+0.702464040915249 < self.epsilon)
          self.assertTrue(n_coe-0.023881520674723 < self.epsilon)
          self.assertTrue(np.abs(n_cse) < self.epsilon)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue May  9 11:11:15 2017

@author: msuzen

%load_ext autoreload
%autoreload 2

"""

import unittest
from bristol.ensembles import Circular
import numpy as np

class test_n_eigen_circular(unittest.TestCase):

      epsilon = 1e-9

      def test_n_eigen_circular_01(self):
          mseed   = 2963416
          ce      = Circular()
          e_cue   = ce._n_eigen_circular(
                                         seed=mseed, 
                                         N=5,
                                         size=1,
                                         ensemble='CUE', 
                                         set_seed=True
                                        )  
          e_coe   = ce._n_eigen_circular(
                                         seed=mseed, 
                                         N=10,
                                         size=1,
                                         ensemble='COE', 
                                         set_seed=True
                                        )   
          e_cse   = ce._n_eigen_circular(
                                         seed=mseed, 
                                         N=15,
                                         size=1,
                                         ensemble='CSE', 
                                         set_seed=True
                                        )   
          n_cue   = np.imag(e_cue['c_eigen']).sum()
          n_coe   = np.imag(e_coe['c_eigen']).sum()
          n_cse   = np.imag(e_cse['c_eigen']).sum()
          self.assertTrue(n_cue-0.70246404091524939 < self.epsilon)
          self.assertTrue(n_coe-0.02388152067472389 < self.epsilon)
          self.assertTrue(np.abs(n_cse)< self.epsilon)
          
          
      def test_n_eigen_circular_02(self):
          mseed   = 2963416
          ce      = Circular()
          e_cue   = ce._n_eigen_circular(
                                         seed=mseed, 
                                         N=5,
                                         size=3,
                                         ensemble='CUE', 
                                         set_seed=True
                                        )
          e_coe   = ce._n_eigen_circular(
                                         seed=mseed, 
                                         N=10,
                                         size=3,
                                         ensemble='COE', 
                                         set_seed=True
                                        )   
          e_cse   = ce._n_eigen_circular(
                                         seed=mseed, 
                                         N=15,
                                         size=3,
                                         ensemble='CSE', 
                                         set_seed=True
                                        )   
          n_cue   = np.imag(e_cue['c_eigen']).sum()
          n_coe   = np.imag(e_coe['c_eigen']).sum()
          n_cse   = np.imag(e_cse['c_eigen']).sum()
          self.assertTrue(n_cue+2.1073921227457482 < self.epsilon)
          self.assertTrue(n_coe-0.0716445620241715 < self.epsilon)
          self.assertTrue(np.abs(n_cse) < self.epsilon)   



import unittest
from bristol.ensembles import Circular
import numpy as np

class test_gen_cse(unittest.TestCase):

      epsilon = 1e-9

      def test_gen_cse_01(self):
          N     = 8
          ce    = Circular()
          mseed = 2963416
          Hcse0 = ce.gen_cse(8, seed=mseed, set_seed=True)
          Hcse1 = ce.gen_cse(8, seed=mseed, set_seed=True)
          n0    = float(np.imag(Hcse0[0,:].sum()))-0.001106565421140888
          n1    = float(np.real(Hcse1[6,:].sum()))-0.021065228262225025
          self.assertTrue(n0 < self.epsilon)
          self.assertTrue(n1 < self.epsilon)

      def test_gen_cse_02(self):
          N      = 8
          ce     = Circular()
          mseed  = 2963416
          Hcse0  = ce.gen_cse(8, seed=mseed, set_seed=True, adir="upper")
          Hcse1  = ce.gen_cse(8, seed=mseed, set_seed=True, adir="upper")
          n0     = float(np.imag(Hcse0[0,:].sum()))-0.001106565421140888
          n1     = float(np.real(Hcse1[6,:].sum()))-0.021065228262225025
          self.assertTrue(n0 < self.epsilon)
          self.assertTrue(n1 < self.epsilon)



import unittest
from bristol.ensembles import Circular
from bristol.spectral import Ergodicity
import numpy as np

class test_kl_distance_symmetric(unittest.TestCase):

      epsilon = 1e-9

      def test_kl_distance_symmetric_01(self):
          ergo     = Ergodicity()
          np.random.seed(1235)
          Nk       = np.random.random(100)
          Nk_minus = np.random.random(100)
          kl       = ergo.kl_distance_symmetric(Nk,Nk_minus)
          self.assertTrue(kl-91.808228136131163 < self.epsilon)



from setuptools import setup

fp = open('./bristol/version.py')
exec(fp.read())

with open('README.md') as f:
    long_description = f.read()

setup(
      name='bristol',
      version=__version__,
      description="Bristol implements techniques developed by Mezzadri with parallel processing capabilities and a data model for further processing for generating random matrices. Circular module provides methods to generate matrices from Circular Unitary Ensemble (CUE), Circular Ortogonal Ensemble (COE) and Circular Symplectic Ensemble (CSE). Additional spectral analysis utilities are also implemented, such as computation of spectral density and spectral ergodicity.",
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="https://github.com/msuzen/bristol",
      author='M.Suzen',
      author_email='suzen@acm.org',
      license= 'GPL-3',
      packages=['bristol'],
      install_requires=['numpy >= 1.11', 'future >= 0.16.0'],
      test_suite="test",
      zip_safe=False
     )



import unittest
from bristol.ensembles import Circular
import numpy as np

class test_gen_coe(unittest.TestCase):

      epsilon = 1e-9

      def test_gen_coe_01(self):
          ce    = Circular()
          mseed = 2963416
          Hcoe0 = ce.gen_coe(8, seed=mseed, set_seed=True)
          Hcoe1 = ce.gen_coe(8, seed=mseed, set_seed=True)
          n0    =  float(np.imag(Hcoe0[0,:].sum()))-0.2712959924904533
          n1    =  float(np.real(Hcoe1[6,:].sum()))+0.4453698237585775
          self.assertTrue(n0 < self.epsilon)
          self.assertTrue(n1 < self.epsilon)
